//=============================================================================
//                                                              SynergyEA.mq4
//                                              Copyright  2007, Derk Wehler
//
//=============================================================================
#property copyright "Copyright  2007, Derk Wehler"
#property link      "derkwehler@gmail.com"

#include <LibOrderReliable.mqh>
#include <LibDerksUtils.mqh>


// Input parameters
extern string	S1 					= "== MONEY MANAGEMENT SETTINGS ==";
extern bool 	UseMoneyManagement 	= false;
extern bool		UseDualWinMM		= false;
extern double	TradeSizePercent 	= 3.0;	// Change to whatever percent of equity you wish to risk.
extern double	ScaleInPercent	 	= 25.0;
extern int		MaxScaleIns			= 0;
extern bool		SplitOrder			= true;
extern double	Lots 				= 0.2;
extern double	MinLots 			= 0.01;
extern double	MaxLots 			= 100.0;
extern string	S1z 				= " ";
extern string	S2 					= "==== ENTRY SETTINGS ====";
extern int		UseEntry68_32		= 0;
extern int		ReqRedYellowCombo	= 0;
extern int		UseVolExpanding		= 0;
extern double	UnlessVolWidthGT	= 0.0;
extern int		ReqEntryNearPAC		= 1;
extern int		UseHTF_Trend		= 0;
extern string	S2z 				= " ";
extern string	S3 					= "==== EXIT SETTINGS ====";
extern int		UseSmallerOrOpp		= 0;
extern int		UseSmallerExit		= 0;
extern double	DefineSmaller		= 0.08;
extern int		UseClosedInside		= 0;
extern int		UseTDI_ExitDir		= 0;
extern int		UseOppPAC_Trail		= 1;
extern int		OppPACforInitSL		= 0;
extern string	S3a 				= "-------------- ATR Exit --------------";
extern bool		UseATR_Exit			= false;
extern int 		ATR_Period			= 5;
extern double 	ATR_PipMult 		= 4;	// try 2.0 - 15.0
extern int 		ATR_MinPips			= 50;
extern int 		ATR_MaxPips			= 500;
extern string	S3b 				= "------ SL, TP & Trail ------";
extern int		StopLoss	 		= 0;
extern int		TakeProfit	 		= 100;
extern int	 	TrailingType 		= 3;
extern int	 	TrailingValue 		= 0;
extern int	 	FirstMove 			= 100;
extern int	 	FirstSL		 		= 100;
extern int	 	SecondMove 			= 0;
extern int	 	SecondSL	 		= 0;
extern int	 	ThirdMove 			= 0;
extern string	S4 					= "==== EA SETTINGS ====";
extern string	S4a 				= "------ Other Settings ------";
extern bool		UseAlert			= false;
extern bool		UseEmail			= false;
extern string	S4b 				= "------ General EA ------";
extern int		Slippage			= 1;
extern int		MagicSeed			= 29000;
extern string	ExpertName			= "SynergyEA";
extern int		DebugLevel 			= 5;


int 	MagicNumber;
double 	LotsMM, LastLotsL, LastLotsS;
double	Spread;
int		ScaleIns = 0;
int		AdjStopLoss;
bool	OkToAddPositions = true;
bool	HA_ValuesAcquired = false;
bool	TDI_ValuesAcquired = false;
bool	PAC_ValuesAcquired = false;

bool InBuy  = false;
bool InSell = false;


// Indicator values
double	HA_Open_0, HA_Close_0;
double 	HA_Open_1, HA_Close_1;
double 	HA_Open_2, HA_Close_2;
double	HA_DM0_2, HA_DM1_2;
double	BlueH_0, Yellow_0, BlueL_0, Green_0, Red_0;
double	BlueH_1, Yellow_1, BlueL_1, Green_1, Red_1, Red_2;
double	MA_Hi_0, MA_Lo_0;
double	MA_Hi_1, MA_Lo_1;
double	MA_Hi_2, MA_Lo_2;
string	ExitReason;

//=============================================================================
//	expert initialization function
//=============================================================================
int init()
{
	// Set up magic numbers:
    MagicNumber = MagicSeed + SymbolConst2Val(Symbol())*100 + TimeFrameConst2Val(Period());
	DU_SetErrorLevel(DebugLevel);

	MinLots = MathMax(MinLots, MarketInfo(Symbol(), MODE_MINLOT));
	MaxLots = MathMin(MaxLots, MarketInfo(Symbol(), MODE_MAXLOT));
	return (0);
}
  

//=============================================================================
//	expert deinitialization function
//=============================================================================
int deinit()
{
	return (0);
}


void SetAdjustedStoplossValue(int cmd)
{
	AdjStopLoss = StopLoss;
	
	if (OppPACforInitSL == 0)
		return;
		
	AcquirePAC_Values();

	// Calculate initial stoploss for Price Action Channel trail
	if (cmd == OP_BUY)
	{
		if (UseOppPAC_Trail == 1)
			AdjStopLoss = (Ask - (MA_Lo_1 - Spread)) / Point;
	}
	else if (cmd == OP_SELL)
	{
		if (UseOppPAC_Trail == 1)
			AdjStopLoss = ((MA_Hi_1 + Spread) - Bid) / Point;
	}
}


//=============================================================================
//
// PURPOSE:
//     Calculate the number of lots to trade.
//
// PARAMETERS:
//	   magic: The magic number used to trade (used by DualWin system only)
//
// RETURN VALUE:
//     The number of lots to trade
//
//=============================================================================
double CalcLots(int magic)
{
	double lots = Lots;

	if (UseMoneyManagement)
	{
		lots = CalcLotsGivenRisk(TradeSizePercent, MinLots, MaxLots, AdjStopLoss);
	}
	if (UseDualWinMM)
	{
		if (TakeProfit == 0)
			Alert("ERROR!! : Attempt to use DualWin MM system with zero TP");
		else
			lots = GetDualWinLots(lots, magic, TakeProfit);
	}
	
	return(lots);
}


//=============================================================================
//
// PURPOSE:
//     Check whether to exit positions, based on ATR Indicator
//
//=============================================================================
int CheckExitOnATR()
{ 
   	DU_FuncName = "CheckExitOnATR";

	int 	i, pipsProfit;
	int		openPos  = NumOpenPositions(MagicNumber, OP_BOTH);
	double 	pipCount = (ATR_PipMult/Point) * (iATR(NULL, 0, ATR_Period, 0));
	
	pipCount = MathMax(pipCount, ATR_MinPips);
	pipCount = MathMin(pipCount, ATR_MaxPips);

	for (i=0; i < openPos; i++)
	{
		OrderSelect(i, SELECT_BY_POS);
		if (OrderType() == OP_SELL)
			pipsProfit = (OrderOpenPrice() - Ask) / Point; 
		else 
			pipsProfit = (Bid - OrderOpenPrice()) / Point; 

		if (pipsProfit > pipCount)
		{
			return(OrderType());
		}
	}
	return(-1);
}


//=============================================================================
//
// PURPOSE:
//     Go through possible entry signals one by one, and short-circuit 
//     out early if any fail (more efficient for back-testing)
//
// PARAMETERS:
//	   cmd: The direction to enter trade
//
// RETURN VALUE:
//     True if we should enter, false otherwise
//
//=============================================================================
bool CheckEntrySignals(int cmd)
{
   	DU_FuncName = "CheckEntrySignals";

	if (GetMA_Trend() != cmd) 
		return (false);

	if (GetHA_CloseDir() != cmd) 
		return (false);

	if (GetTDI_EntryDir() != cmd) 
		return (false);

	if (UseVolExpanding == 1)
		if (VolBandsExpanding() == false) 
			return (false);

	if (UseHTF_Trend == 1)
		if (GetTrendHigherTF() != cmd) 
			return (false);

	if (ReqEntryNearPAC == 1)
		if (PriceCloseEnoughToPAC(cmd) == false) 
			return (false);
	
	// If we get to here all rules pass
	return (true);	             
}


//=============================================================================
//
// PURPOSE:
//     Go through possible exit signals one by one, and short-circuit 
//     out early if any are hit (more efficient for back-testing)
//
// PARAMETERS:
//	   cmd: The direction to enter trade
//
// RETURN VALUE:
//     True if we should exit, false otherwise
//
//=============================================================================
bool CheckExitSignals(int cmd)
{
   	DU_FuncName = "CheckExitSignals";

	if (UseSmallerOrOpp == 1)
		if (GetCandleSmallerOrOpposite() == cmd) 
			return (true);

	if (UseClosedInside == 1)
		if (GetHA_ClosedInside() == true) 
			return (true);

	if (UseTDI_ExitDir == 1)
		if (GetTDI_ExitDir() == cmd) 
			return (true);

	if (UseOppPAC_Trail == 1)
		if (PriceReachedOppositePAC() == cmd) 
			return (true);

	return (false);
}


//=============================================================================
//
// PURPOSE:
//     Enter a trade with the given parameters
//
// PARAMETERS:
//	     op: The direction to enter trade
//     lots: The percentage to use if MM is enabled
//
// RETURN VALUE:
//     The ticket number of the trade entered
//
//=============================================================================
int DoEntry(int op, double lots, int slPips, int tpPips)
{
   	DU_FuncName = "DoEntry";

	int 	mult, ticket;
	color	clr;
	double 	price, sl, tp;

	sl = 0;
	tp = 0;
	mult = 1;
	price = Ask;
	clr = Green;
	
	if (op == OP_SELL)
	{
		mult = -1;
		price = Bid;
		clr = Red;
	}
	
	if (slPips > 0)
		sl = price - mult * slPips*Point;
	if (tpPips > 0)
		tp = price + mult * tpPips*Point;
		
	if (IsTesting())
	{
		ticket = OrderSend(Symbol(), op, lots, price, Slippage, sl, tp, 
							ExpertName, MagicNumber, 0, clr);
	}
	else
	{
		ticket = OrderSendReliable(Symbol(), op, lots, price, Slippage, sl, tp, 
									ExpertName, MagicNumber, 0, clr);
	}
	return(ticket);
}


//=============================================================================
//
// PURPOSE:
//     Exit all trades in a given direction
//
// PARAMETERS:
//	     op: The direction to enter trade
//     lots: The percentage to use if MM is enabled
//
// RETURN VALUE:
//     true: A trdae was exited
//    false: No trade exited
//
//=============================================================================
bool DoExit(int op)
{
   	DU_FuncName = "DoExit";

	int		cnt;
	bool	somethingClosed = false;
	color	clr;
	double 	price, sl, tp;

	if (op != OP_BUY && op != OP_SELL)
		return(false);
		
	for (cnt=OrdersTotal()-1; cnt >= 0; cnt--)
	{
		OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
		if (OrderSymbol() != Symbol() || 
			OrderMagicNumber() != MagicNumber || 
			OrderType() != op) 
			continue;
		
		price = Bid;
		clr = Green;
	
		if (op == OP_SELL)
		{
			price = Ask;
			clr = Red;
		}
	
		if (IsTesting())
		{
			if (OrderClose(OrderTicket(), OrderLots(), price, Slippage, clr))
				somethingClosed = true;
		}
		else
		{
			if (OrderCloseReliable(OrderTicket(), OrderLots(), price, Slippage, clr))
				somethingClosed = true;
		}
	}
	
	if (somethingClosed)
	{
		if (op == OP_BUY)
			InBuy = false;
		else
			InSell = false;
	}
	return(somethingClosed);
}


//=============================================================================
//	Expert start function
//=============================================================================
int start()
{
   	DU_FuncName = "Synergy start()";

	int 	ticket, openPos, exitPos;
	double	lotsMM;
	
	// Set up some vars based on whether we're in a trade
	InBuy  = false;
	InSell = false;
	if (NumOpenPositions(MagicNumber, OP_BUY) > 0)
		InBuy  = true;
	if (NumOpenPositions(MagicNumber, OP_SELL) > 0)
		InSell = true;
	if (!InBuy)		LastLotsL = 0;
	if (!InSell)	LastLotsS = 0;
	
	// Handle triling stop on every pip
	if (TrailingType > 0 && NumOpenPositions(MagicNumber, OP_BOTH) > 0)
		AdjTrailOnAllOrders(TrailingType, TrailingValue, MagicNumber, OP_BOTH, 
							FirstMove, FirstSL, SecondMove, SecondSL, ThirdMove);

	// Handle variable TP exit
	if (UseATR_Exit)
	{
		exitPos = CheckExitOnATR();
		if (exitPos != -1)
		{
			DoExit(exitPos);
			DU_Log(1, "", "");
			DU_Log(1, "", "                   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
			DU_Log(1, "", "                                                     EXITING ON ATR         ");
			DU_Log(1, "", "                   <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<");
			DU_Log(1, "", "");
		}
	}

	// Check for exits once per candle
	Spread = Ask - Bid;
	if (IsNewCandle())
	{
		OkToAddPositions = true;
		
		// Check for exit signals --------------------------------------------
		
		// If all in Short position, signal to EXIT LONG at this candle's open
		if (InBuy)
		{
			if (CheckExitSignals(OP_BUY))
			{
				DU_Log(1, "", "                                      Got EXIT BUY signal: " + ExitReason);
				DoAlert("Synergy: Exit Buy Signal: " + ExitReason);
				DoExit(OP_BUY);
			}
		}
					
		// If all in Long position, signal to EXIT SHORT position at this candle's open
		if (InSell)
		{
			if (CheckExitSignals(OP_SELL))
			{
				DU_Log(1, "", "                                      Got EXIT SELL signal: " + ExitReason);
				DoAlert("Synergy: Exit Sell Signal: " + ExitReason);
				DoExit(OP_SELL);
			}
		}
	}
		

	// Check for entry signals ---------------------------------------------
	
	// If all in Long position, signal to BUY at this candle's open
	if (!InBuy)
	{
		if (CheckEntrySignals(OP_BUY))
		{
			// First make sure we are not in any opposite trades
			DoExit(OP_SELL);

			SetAdjustedStoplossValue(OP_BUY);	// Calculate initial stoploss
			lotsMM = CalcLots(MagicNumber);		// Calc the lots to use
			LastLotsL = lotsMM;

			// Open new position(s)
			if (SplitOrder)
			{
				lotsMM = NormalizeDouble(lotsMM/2, 2);
				DoEntry(OP_BUY, lotsMM, AdjStopLoss, 0);
			}
			ticket = DoEntry(OP_BUY, lotsMM, AdjStopLoss, TakeProfit);

			if (ticket != -1)
				InBuy = true;
			DoAlert("SynergyEA: New Buy Signal");
		}
	}
	else if (InBuy)
	{
		if (AddToBuyPosition())
		{
			// Add to position if we already have one open in this direction
			openPos = NumOpenPositions(MagicNumber, OP_BUY);
			if (openPos > 0 && openPos <= MaxScaleIns && OkToAddPositions)
			{
				SetAdjustedStoplossValue(OP_BUY);	// Calculate initial stoploss
				lotsMM = NormalizeDouble((LastLotsL * ScaleInPercent / 100), 2);
				DoEntry(OP_BUY, lotsMM, AdjStopLoss, TakeProfit);
				OkToAddPositions = false;
			}

			DoAlert("SynergyEA: Add to Buy Signal");
		}
	}
		
	// If all in Short position, signal to SELL at this candle's open
	if (!InSell)
	{
		if (CheckEntrySignals(OP_SELL))
		{
			// First make sure we are not in any opposite trades
			DoExit(OP_BUY);

			SetAdjustedStoplossValue(OP_SELL);	// Calculate initial stoploss
			lotsMM = CalcLots(MagicNumber);		// Calc the lots to use
			LastLotsS = lotsMM;

			// Open new position(s)
			if (SplitOrder)
			{
				lotsMM = NormalizeDouble(lotsMM/2, 2);
				DoEntry(OP_SELL, lotsMM, AdjStopLoss, 0);
			}
			ticket = DoEntry(OP_SELL, lotsMM, AdjStopLoss, TakeProfit);

			if (ticket != -1)
				InSell = true;
			DoAlert("SynergyEA: New Sell Signal");
		}
	}	
	else if (InSell)
	{
		if (AddToSellPosition())
		{
			// Add to position if we already have one open in this direction
			openPos = NumOpenPositions(MagicNumber, OP_SELL);
			if (openPos > 0 && openPos <= MaxScaleIns && OkToAddPositions)
			{
				SetAdjustedStoplossValue(OP_SELL);	// Calculate initial stoploss
				lotsMM = NormalizeDouble((LastLotsS * ScaleInPercent / 100), 2);
				DoEntry(OP_SELL, lotsMM, AdjStopLoss, TakeProfit);
				OkToAddPositions = false;
			}

			DoAlert("SynergyEA: Add to Sell Signal");
		}
	}	

	// Reset these for the next run through
	HA_ValuesAcquired = false;
	TDI_ValuesAcquired = false;
	PAC_ValuesAcquired = false;
	return(0);
}


void DoAlert(string str)
{
	if (UseAlert)
		Alert(str);

	if (UseEmail)
		SendMail("SynergyInd", str);
}


void AcquireHA_Values() 
{
	if (HA_ValuesAcquired)
		return;
		
	// Get Haiken Ashi values =========================================================================================
	// Current Candle...
	HA_Open_0 	= iCustom(NULL, 0, "HeikenAshi_DM",   2, 0); // Open
	HA_Close_0 	= iCustom(NULL, 0, "HeikenAshi_DM",   3, 0); // Close
	
	// Last Closed Candle...
	HA_Open_1 	= iCustom(NULL, 0, "HeikenAshi_DM",   2, 1); // Open
	HA_Close_1 	= iCustom(NULL, 0, "HeikenAshi_DM",   3, 1); // Close
	
	// Prev Prev Candle...
	HA_Open_2 	= iCustom(NULL, 0, "HeikenAshi_DM",   2, 2); // Open
	HA_Close_2 	= iCustom(NULL, 0, "HeikenAshi_DM",   3, 2); // Close
	
	HA_ValuesAcquired = true;
}

void AcquireTDI_Values() 
{
	if (TDI_ValuesAcquired)
		return;
		
	// Get Traders Dynamic Index values ===============================================================================
	// Current Candle...
	BlueH_0 	= iCustom(NULL, 0, "Traders_Dynamic_Index", 13, 0, 34, 2, 0, 7, 0,   1, 0);	// Blue   (H. Vol Band)
	Yellow_0 	= iCustom(NULL, 0, "Traders_Dynamic_Index", 13, 0, 34, 2, 0, 7, 0,   2, 0);	// Yellow (Market Base)
	BlueL_0 	= iCustom(NULL, 0, "Traders_Dynamic_Index", 13, 0, 34, 2, 0, 7, 0,   3, 0);	// Blue   (L. Vol Band)
	Green_0 	= iCustom(NULL, 0, "Traders_Dynamic_Index", 13, 0, 34, 2, 0, 7, 0,   4, 0);	// Green  (RSI Price)
	Red_0 		= iCustom(NULL, 0, "Traders_Dynamic_Index", 13, 0, 34, 2, 0, 7, 0,   5, 0);	// Red    (Trade Signal)
	
	// Last Closed Candle...
	BlueH_1 	= iCustom(NULL, 0, "Traders_Dynamic_Index", 13, 0, 34, 2, 0, 7, 0,   1, 1);	// Blue   (H. Vol Band)
	Yellow_1 	= iCustom(NULL, 0, "Traders_Dynamic_Index", 13, 0, 34, 2, 0, 7, 0,   2, 1);	// Yellow (Market Base)
	BlueL_1 	= iCustom(NULL, 0, "Traders_Dynamic_Index", 13, 0, 34, 2, 0, 7, 0,   3, 1);	// Blue   (L. Vol Band)
	Green_1 	= iCustom(NULL, 0, "Traders_Dynamic_Index", 13, 0, 34, 2, 0, 7, 0,   4, 1);	// Green  (RSI Price)
	Red_1 		= iCustom(NULL, 0, "Traders_Dynamic_Index", 13, 0, 34, 2, 0, 7, 0,   5, 1);	// Red    (Trade Signal)
	
	// Prev Prev Candle...
	Red_2 		= iCustom(NULL, 0, "Traders_Dynamic_Index", 13, 0, 34, 2, 0, 7, 0,   5, 2);	// Red    (Prev Prev Candle)
	
	TDI_ValuesAcquired = true;
}
	
void AcquirePAC_Values() 
{
	if (PAC_ValuesAcquired)
		return;
		
	// Get Smoothed Moving Average values =============================================================================
	MA_Hi_0 = iMA(NULL, 0, 5, 0, MODE_SMMA, PRICE_HIGH, 0);
	MA_Lo_0 = iMA(NULL, 0, 5, 0, MODE_SMMA, PRICE_LOW,  0);
	MA_Hi_1 = iMA(NULL, 0, 5, 0, MODE_SMMA, PRICE_HIGH, 1);
	MA_Lo_1 = iMA(NULL, 0, 5, 0, MODE_SMMA, PRICE_LOW,  1);
	MA_Hi_2 = iMA(NULL, 0, 5, 0, MODE_SMMA, PRICE_HIGH, 2);
	MA_Lo_2 = iMA(NULL, 0, 5, 0, MODE_SMMA, PRICE_LOW,  2);
	PAC_ValuesAcquired = true;
}


int GetMA_Trend()
{
	int sum = 0;
	
	AcquirePAC_Values();
	
	// We are looking for the general trend here, so check 
	// whether lines are headed up or down, comparing this 
	// candle with the prev and the prev prev
	if (MA_Hi_0 > MA_Hi_1)	sum++;
	if (MA_Lo_0 > MA_Lo_1)	sum++;
	if (MA_Hi_0 > MA_Hi_2)	sum++;
	if (MA_Lo_0 > MA_Lo_2)	sum++;
	if (MA_Hi_1 > MA_Hi_2)	sum++;
	if (MA_Lo_1 > MA_Lo_2)	sum++;
		
	if (MA_Hi_0 < MA_Hi_1)	sum--;
	if (MA_Lo_0 < MA_Lo_1)	sum--;
	if (MA_Hi_0 < MA_Hi_2)	sum--;
	if (MA_Lo_0 < MA_Lo_2)	sum--;
	if (MA_Hi_1 < MA_Hi_2)	sum--;
	if (MA_Lo_1 < MA_Lo_2)	sum--;

	// If we have 3 (or 4??) out of 6, consider it to have a direction
	if (sum >= 3)
		return(OP_BUY);
	else if (sum <= -3)
		return(OP_SELL);
	else
		return(-1);
}


int GetTrendHigherTF()
{
	// Get Higher Timeframe Haiken Ashi values =====================================
	int higherTF;
	switch (Period())
	{
		case PERIOD_M1:		higherTF = PERIOD_M5;	break;
		case PERIOD_M5:		higherTF = PERIOD_M15;	break;
		case PERIOD_M15:	higherTF = PERIOD_M30;	break;
		case PERIOD_M30:	higherTF = PERIOD_H1;	break;
		case PERIOD_H1:		higherTF = PERIOD_H4;	break;
		case PERIOD_H4:		higherTF = PERIOD_D1;	break;
		case PERIOD_D1:		higherTF = PERIOD_W1;	break;
		default:			higherTF = PERIOD_M1;	break;
	}
	
	// Current Candle...
	double HA_Open0 	= iCustom(NULL, higherTF, "HeikenAshi_DM",   2, 0); // Open
	double HA_Close0 	= iCustom(NULL, higherTF, "HeikenAshi_DM",   3, 0); // Close
	
	// Last Closed Candle...
	double HA_Open1 	= iCustom(NULL, higherTF, "HeikenAshi_DM",   2, 1); // Open
	double HA_Close1 	= iCustom(NULL, higherTF, "HeikenAshi_DM",   3, 1); // Close
	
	// Prev Prev Candle...
	double HA_Open2 	= iCustom(NULL, higherTF, "HeikenAshi_DM",   2, 2); // Open
	double HA_Close2 	= iCustom(NULL, higherTF, "HeikenAshi_DM",   3, 2); // Close
	


	// Now check for 3 in a row being the same =====================================
	int		dir0 = -1;
	int		dir1 = -1;
	int		dir2 = -1;
	
	if (HA_Open0 < HA_Close0) 		dir0 = OP_BUY;
	else 
	if (HA_Open0 > HA_Close0)		dir0 = OP_SELL;
		
	if (HA_Open1 < HA_Close1) 		dir1 = OP_BUY;
	else 
	if (HA_Open1 > HA_Close1)		dir1 = OP_SELL;
		
	if (HA_Open2 < HA_Close2) 		dir2 = OP_BUY;
	else 
	if (HA_Open2 > HA_Close2)		dir2 = OP_SELL;
	
	if (dir0 == OP_BUY && dir1 == OP_BUY && dir2 == OP_BUY)		return(OP_BUY);
	else 
	if (dir0 == OP_SELL && dir1 == OP_SELL && dir2 == OP_SELL)	return(OP_SELL);

	return(-1);
}


int GetHA_CloseDir()
{
	AcquireHA_Values();
	AcquirePAC_Values();
		
	if (HA_Close_0 > HA_Open_0 && HA_Close_0 > MA_Hi_0)
	{
		return(OP_BUY);
	}
	else 
	if (HA_Close_0 < HA_Open_0 && HA_Close_0 < MA_Lo_0)
	{
		return(OP_SELL);
	}
	return(-1);
}


bool VolBandsExpanding()
{
   	DU_FuncName = "VolBandsExpanding";

	AcquireTDI_Values();

	if (BlueH_0 - BlueL_0 > BlueH_1 - BlueL_1)
	{
		DU_Log(6, "", "");
		DU_Log(6, "", "             BlueH_0: " + BlueH_0 + ", BlueL_0: " + BlueL_0 + "    BlueH_1: " + BlueH_1 + ", BlueL_1: " + BlueL_1);
		DU_Log(6, "", "             Volume EXPANDING (" + (BlueH_0 - BlueL_0) + " vs. prev: " + (BlueH_1 - BlueL_1) + ")");
		return(true);
	}
	if (UnlessVolWidthGT != 0 && (BlueH_0 - BlueL_0) > UnlessVolWidthGT)
	{
		DU_Log(6, "", "             Volume NOT expanding (" + (BlueH_0 - BlueL_0) + " vs. prev: " + (BlueH_1 - BlueL_1) + "), but still greater than " + UnlessVolWidthGT);
		return(true);
	}
	return(false);
}


int GetTDI_EntryDir()
{
	AcquireTDI_Values();

	if (Green_0 > 50.0 && 
		(Green_0 < 68.0  || !UseEntry68_32) && 
		(Green_0 > Yellow_0 && Green_0 > Red_0) && 
		(Red_0 > Red_1) && //Red_1 > Red_2) && 
		(Red_0 > Yellow_0 || !ReqRedYellowCombo))
		return(OP_BUY);
	else 
	if (Green_0 < 50.0 && 
		(Green_0 > 32.0 || !UseEntry68_32) && 
		(Green_0 < Yellow_0 && Green_0 < Red_0) && 
		(Red_0 < Red_1) && //Red_1 < Red_2) && 
		(Red_0 < Yellow_0 || !ReqRedYellowCombo))
		return(OP_SELL);
		
	return(-1);
}


bool PriceCloseEnoughToPAC(int cmd)
{
	AcquirePAC_Values();
	double PAC_Dist = MA_Hi_0 - MA_Lo_0;
	
	if (cmd == OP_BUY)
	{
		if (Ask - MA_Hi_0 > PAC_Dist)
			return(false);
	}
	else if (cmd == OP_SELL)
	{
		if (MA_Lo_0 - Bid > PAC_Dist)
			return(false);
	}
	return(true);
}


int GetTDI_ExitDir()
{
	AcquireTDI_Values();

	// Check for exiting Buy position
	if (Green_1 > 68.0 && Green_1 < BlueH_1 && Green_1 < Red_1)
	{
		ExitReason = "TDI";
		return(OP_BUY);
	}
	else 
	// Check for exiting sell position
	if (Green_1 < 32.0 && Green_1 > BlueL_1 && Green_1 > Red_1)
	{
		ExitReason = "TDI";
		return(OP_SELL);
	}
	return(-1);
}


bool GetHA_ClosedInside()
{
	AcquireHA_Values();
	AcquirePAC_Values();
		
	// HA_Close_1 is close price
	if (HA_Close_1 > MA_Lo_1 && HA_Close_1 < MA_Hi_1)
	{
		ExitReason = "CI";
		return(true);
	}
	else 
		return(false);
}


int GetCandleSmallerOrOpposite()
{
	AcquireHA_Values();
		
	if (HA_Close_2 > HA_Open_2)		// 2 candles back was long
	{
		// If prev candle was short, return OP_BUY signal to exit buy
		if (HA_Close_1 < HA_Open_1)
		{
			ExitReason = "OC";
			return(OP_BUY);
		}
			
		// If prev candle was "much smaller" than prev prev, then exit
		if (UseSmallerExit == 1)
		{
			if (HA_Close_1 - HA_Open_1 < (HA_Close_2 - HA_Open_2) * DefineSmaller)
			{
				ExitReason = "SC";
				return(OP_BUY);
			}
		}
	}
	else if (HA_Close_2 < HA_Open_2)	// 2 candles back was short
	{
		// If prev candle was long, return OP_SELL signal to exit sell
		if (HA_Close_1 > HA_Open_1)
		{
			ExitReason = "OC";
			return(OP_SELL);
		}
					
		// If prev candle was "much smaller" than prev prev, then exit
		if (UseSmallerExit == 1)
		{
			if (HA_Open_1 - HA_Close_1 < (HA_Open_2 - HA_Close_2) * DefineSmaller)
			{
				ExitReason = "SC";
				return(OP_SELL);
			}
		}
	}
	return(-1);
}


int PriceReachedOppositePAC()
{
	AcquirePAC_Values();
		
	if (Close[1] >= MA_Hi_1 + Spread)
	{
		ExitReason = "PAC";
		return(OP_SELL);	// return OP_SELL signal to exit sell
	}
	
	if (Close[1] <= MA_Lo_1 - Spread)
	{
		ExitReason = "PAC";
		return(OP_BUY);		// return OP_BUY signal to exit buy
	}
	return(-1);
}


bool AddToBuyPosition()
{
	AcquireTDI_Values();
		
	if (Green_1 > 50.0 && Green_1 < 68.0 && Green_1 > BlueH_1)
		return(true);
	return(false);
}


bool AddToSellPosition()
{
	AcquireTDI_Values();
		
	if (Green_1 < 50.0 && Green_1 > 32.0 && Green_1 < BlueL_1)
		return(true);
	return(false);
}


